library Pascal;


uses
  Windows,
  SysUtils,
  Classes,
  ifpscomp,
  ifps3,
  ifpidll2,
  ifpidll2runtime,
  ifpiir_std,
  ifpii_std,
  ifpidateutils,
  ifpidateutilsr,
  ifpiir_classes,
  ifpii_classes,
  NSIS,
  Utils in 'Utils.pas';


function ScriptOnUses(Sender: TIFPSPascalCompiler; const Name: string): Boolean;
begin
  if Name = 'SYSTEM' then
  begin
    RegisterNSISLibrary_C(Sender);
    Sender.OnExternalProc := @DllExternalProc;
    RegisterDateTimeLibrary_C(Sender);
    SIRegister_Std(Sender);
    SIRegister_Classes(Sender, True);
    Result := True;
  end else
    Result := False;
end;

function ExecuteScript(const Script: string): Boolean;
var
  Compiler: TIFPSPascalCompiler;
  { TIFPSPascalCompiler is the compiler part of the scriptengine. This will
    translate a Pascal script into a compiled for the executer understands. }
  Exec: TIFPSExec;
   { TIFPSExec is the executer part of the scriptengine. It uses the output of
    the compiler to run a script. }
  Data: string;
  CI: TIFPSRuntimeClassImporter; // Can be found in the ifpiclassruntime.pas unit.
  C: Integer;
  ErrorMessages: String;
begin
  Result := False;
  Compiler := TIFPSPascalCompiler.Create; // create an instance of the compiler.
  Compiler.OnUses := ScriptOnUses; // assign the OnUses event.
  if not Compiler.Compile(Script) then  // Compile the Pascal script into bytecode.
  begin
    ErrorMessages := '';
    for C := 0 to Compiler.MsgCount - 1 do
      ErrorMessages := ErrorMessages + Compiler.Msg[C].MessageToString + #13;
       MessageBox(g_hwndParent, PChar(ErrorMessages), 'Pascal error', MB_OK or MB_ICONHAND);
    Compiler.Free;
    Exit;
  end;

  Compiler.GetOutput(Data); // Save the output of the compiler in the string Data.
  Compiler.Free; // After compiling the script, there is no need for the compiler anymore.

  CI := TIFPSRuntimeClassImporter.Create;
  { Create an instance of the runtime class importer.}

  Exec := TIFPSExec.Create;  // Create an instance of the executer.
  RegisterNSISLibrary_R(Exec);
  RegisterDateTimeLibrary_R(Exec);
  RegisterDLLRuntime(Exec);

  RIRegister_Std(CI);
  RIRegister_Classes(CI, True);

  RegisterClassLibraryRuntime(Exec, CI);
  // Assign the runtime class importer to the executer.

  if not Exec.LoadData(Data) then // Load the data from the Data string.
  begin
    { For some reason the script could not be loaded. This is usually the case when a
      library that has been used at compile time isn't registered at runtime. }
    Exec.Free;
     // You could raise an exception here.
    Exit;
  end;

  Exec.RunScript; // Run the script.
  Exec.Free; // Free the executer.
  CI.Free;
  Result := True;
end;


function Script(hwndParent: HWND; string_size: integer; variables: PChar; stacktop: pointer):integer; cdecl;
var
  P: PChar;
begin
  Init(hwndParent,string_size,variables,stacktop);
  Result := 0;
  GetMem(P, string_size);
  try
    if ((PopString(P) = 0) and ExecuteScript(P)) then
      PushString('success')
    else begin
      PushString('error');
      Result := 1;
    end;
  finally
    FreeMem(P);
  end;
end;

function ScriptFile(hwndParent: HWND; string_size: integer; variables: PChar; stacktop: pointer):integer; cdecl;
var
  FileName: array[0..MAX_PATH] of Char;
begin
  Init(hwndParent,string_size,variables,stacktop);
  Result := 0;
  PopString(FileName);
  try
    with TStringList.Create do
    try
      LoadFromFile(FileName);
      if not ExecuteScript(Text) then Abort;
      PushString('success');
    finally
      Free;
    end;
  except
    PushString('error');
    Result := 1;
  end;
end;

exports
  Script, ScriptFile;

begin
end.
